Package: rust
Type: Package
Title: Ratio-of-Uniforms Simulation with Transformation
Version: 1.4.3
Date: 2024-08-14
Authors@R: person(c("Paul", "J."), "Northrop", email = "p.northrop@ucl.ac.uk",
  role = c("aut", "cre", "cph"))
Description: Uses the generalized ratio-of-uniforms (RU) method to simulate
    from univariate and (low-dimensional) multivariate continuous distributions.
    The user specifies the log-density, up to an additive constant. The RU
    algorithm is applied after relocation of mode of the density to zero, and
    the user can choose a tuning parameter r. For details see Wakefield, Gelfand
    and Smith (1991) <DOI:10.1007/BF01889987>, Efficient generation of random
    variates via the ratio-of-uniforms method, Statistics and Computing (1991)
    1, 129-133.  A Box-Cox variable transformation can be used to make the input
    density suitable for the RU method and to improve efficiency.  In the
    multivariate case rotation of axes can also be used to improve efficiency.
    From version 1.2.0 the 'Rcpp' package 
    <https://cran.r-project.org/package=Rcpp> can be used to improve efficiency.
Imports: graphics, Rcpp (>= 0.12.10), stats
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 3.3.0)
RoxygenNote: 7.2.3
Suggests: bang, knitr, microbenchmark, revdbayes, rmarkdown, testthat
VignetteBuilder: knitr
URL: https://paulnorthrop.github.io/rust/,
        https://github.com/paulnorthrop/rust
BugReports: https://github.com/paulnorthrop/rust/issues
LinkingTo: Rcpp (>= 0.12.10), RcppArmadillo
Config/testthat/edition: 3
Author: Paul J. Northrop [aut, cre, cph]
Maintainer: Paul J. Northrop <p.northrop@ucl.ac.uk>
Built: R 4.4.1; x86_64-apple-darwin19.6.0; 2024-08-17 19:55:55 UTC; unix
