# Generated by roxygen2: do not edit by hand

S3method(plot,ru)
S3method(print,ru)
S3method(print,summary.ru)
S3method(summary,ru)
export(create_log_j_xptr)
export(create_phi_to_theta_xptr)
export(create_xptr)
export(find_lambda)
export(find_lambda_one_d)
export(find_lambda_one_d_rcpp)
export(find_lambda_rcpp)
export(gpd_init)
export(gpd_logpost)
export(gpd_sum_stats)
export(rgpd)
export(ru)
export(ru_rcpp)
importFrom(Rcpp,sourceCpp)
importFrom(stats,runif)
useDynLib(rust, .registration = TRUE)
